/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>
#include <stdio.h> 


// The core of a MIDI plugin
void Module::OnMidiData( unsigned long p_clock, unsigned long midi_msg, short pin_id)
{
	// system messages are for timing info etc
	// they don't have a channel, just pass them right through
	bool is_system_msg = ( midi_msg & SYSTEM_MSG ) == SYSTEM_MSG;

	if( !is_system_msg )
	{
		// The first byte of a midi message contains the status (note on/off etc) and the channel
		int chan = midi_msg & 0x0f;
		int stat = midi_msg & 0xf0;

		// the next two bytes depend on the type of message
		int b2 = ( midi_msg / 0x100 ) & 0xff;
		int b3 = ( midi_msg / 0x10000 ) & 0xff;

		switch( stat )
		{
		case NOTE_ON:
		case NOTE_OFF:
			{
				b2 += transpose_ammount; // byte 2 is the note number
				thisP =  (((float) transpose_ammount)  / 120.0f);
			}

			// re-assemble the MIDI message into an 32 bit integer
			if((b2 <= 127)&&(b2 >= 0)) midi_msg = stat + chan + (b2 << 8) + (b3 << 16);
			break;
		}
	}

	// send the MIDI
	getPin(PN_MIDI_OUT)->TransmitMIDI( p_clock, midi_msg );
}

void Module::sub_process(long buffer_offset, long sampleFrames )
{
	
	float *out    = buffer_offset + output1_buffer;
	

	while(--sampleFrames >= 0)
	{
    	               
       
		*out = thisP;
		out++;
	}
}


Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::open()
{
	SEModule_base::open();
    thisP =  (((float) transpose_ammount)  / 120.0f);
	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);
	
}

Module::~Module()
{
	// nothing to see here, move along.
}

// describe the module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	
   LL dish;
   BB bish;
   sprintf_s(dish.thisSEM   ,100, "%s\n",  "MIDI_transPOUT_M" );
   int j;

   j  = sprintf_s(  bish.buffer    ,1000, "%s\n", " ************************************" );
   j  = sprintf_s(  bish.buffer + j,1000, "%s\n", " * 2009 December in Tbingen GERMANY " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *            FAKE MUSIC             " ); 
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *         by ralph.a.cronin         " ); 
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );
   j += sprintf_s(  bish.buffer + j,1000, "%s"  ,   dish.thisSEM );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " The MIDI_IN gets offsetted by Transpose (ListEntry)");
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " to the Midi OUT, the Pout value shows the amount of it");
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " acomodate with 1 Volt is an Oktave of transpose.");
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " Notes out of range <0 and >127 are supressed!");
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *          enjoy SYNTHEDIT          " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *        by Jeff Mc Clintock        " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );

	properties->name  = dish.thisSEM;		
	properties->id    = dish.thisSEM;
	properties->about = bish.buffer;
	return true;


}

// describe the pins (plugs)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )
	{
	case 0:
		properties->name				= "MIDI In";
		properties->direction			= DR_IN;
		properties->datatype			= DT_MIDI2;
		break;
	case 1:
		properties->name				= "Transpose";
		properties->variable_address	= &transpose_ammount;
		properties->datatype_extra		= "range -128,128";
		properties->default_value		= "0";
		properties->direction			= DR_IN;
		properties->datatype			= DT_ENUM;
		break;
	case 2:
		properties->name				= "MIDI Out";
		properties->direction			= DR_OUT;
		properties->datatype			= DT_MIDI2;
		break;
	case 3:
		properties->name				= "POUT";
        properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}


void Module::OnPlugStateChange(SEPin *pin)
{
	 getPin(PN_MIDIPITCH)->TransmitStatusChange( SampleClock(), ST_RUN);// statt out_stat ralph
	

}